# $Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Helper-UDP/lib/Fuzzled/Helper/UDP.pm,v 1.2 2007-07-02 21:59:31 timb Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# (c) Tim Brown, 2006
# <mailto:timb@nth-dimension.org.uk>
# <http://www.nth-dimension.org.uk/> / <http://www.machine.org.uk/>

package Fuzzled::Helper::UDP;

use 5.008007;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

use IO::Socket::INET;
use IO::Select;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration use Fuzzled::Helper::UDP ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = ('$Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Helper-UDP/lib/Fuzzled/Helper/UDP.pm,v 1.2 2007-07-02 21:59:31 timb Exp $' =~ m/.Header: .* ([\d.]+) .*/)[0];

# Preloaded methods go here.

sub new {
	my $self;
	my $hostname;
	my $portnumber;
	$self = shift;
	$self = bless({}, $self);
	$hostname = shift;
	$portnumber = shift;
	$self->{'debugflag'} = shift;
	$self->{'sockethandle'} = IO::Socket::INET->new(PeerAddr => $hostname, PeerPort => $portnumber, Proto => "udp");
	if ($self->{'sockethandle'}) {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::UDP->new() succeeded: connected to host " . $hostname . ", port " . $portnumber . "\r\n";
		return $self;
	} else {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::UDP->new() failed: unable to connect to host " . $hostname . ", port " . $portnumber . "\r\n";
		return undef;
	}
}

sub read {
	my $self;
	my $readbuffer;
	my $readdata;
	$self = shift;
	$readdata = "";
	sysread($self->{'sockethandle'}, $readbuffer, 65535);
	$readdata = $readbuffer;
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::UDP->read() succeeded: read " . length($readdata) . " byte(s)\r\n";
	return $readdata;
}

sub write {
	my $self;
	my $writedata;
	my $returncode;
	$self = shift;
	$writedata = shift;
	$returncode = syswrite($self->{'sockethandle'}, $writedata);
	if ($returncode) {
		if ($returncode = length($writedata)) {
			$self->{'debugflag'} && print STDERR "Fuzzled::Helper::UDP->write() succeeded: wrote " . $returncode . " byte(s)\r\n";
			return 1;
		} else {
			$self->{'debugflag'} && print STDERR "Fuzzled::Helper::UDP->write() failed: wrote " . $returncode . " of " . length($writedata) . " byte(s)\r\n";
			return 0;
		}
	} else {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::UDP->write() failed\r\n";
		return 0;
	}
}

sub eof() {
	my $self = shift;
	if ($self->{'sockethandle'}->eof() == 1) {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::TCP->eof() succeeded: eof\n";
		return 1;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::TCP->eof() succeeded\n";
	return 0;
}

sub close {
	my $self = shift;
	close($self->{'sockethandle'});
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::UDP->close() succeeded: disconnected from host\r\n";
	return 1;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Fuzzled::Helper::UDP - Perl extension for Fuzzled to handle the UDP connection

=head1 SYNOPSIS

  use Fuzzled::Helper::UDP;

  my $udptesthandle;
  my $readdata;

  $udptesthandle = Fuzzled::Helper::UDP->new("service.example.org", 9999, 1);
  $udptesthandle->write("HELLO");
  $readdata = $udptesthandle->read();
  $udptesthandle->close();

=head1 DESCRIPTION

C<Fuzzled::Helper::UDP> is a Perl extension for Fuzzled to handle the UDP connection.  The following methods are available:

=over 4

=item Fuzzled::Helper::UDP->new(<hostname>, <portnumber>, <debugflag>)

Constructs a new C<Fuzzled::Helper::UDP> object connecting to the host and port to be fuzzed.

=item <readdata> = Fuzzled::Helper::UDP->read()

Reads from the socket.

Returns the read data.

=item Fuzzled::Helper::UDP->write(<writedata>)

Writes to the socket.

Returns 1 on success and 0 on failure.

=item Fuzzled::Helper::UDP->eof()

Returns 1 on EOF and 0 in other cases.

=item Fuzzled::Helper::UDP->close()

Closes the connection.

Returns 1.

=head1 SEE ALSO

=head1 AUTHOR

(c) Tim Brown, 2006
<mailto:timb@nth-dimension.org.uk>

=head1 COPYRIGHT AND LICENSE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

=cut