# $Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-IMAP/lib/Fuzzled/Protocol/IMAP.pm,v 1.6 2007-09-15 22:14:32 timb Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with self program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# (c) Tim Brown, 2007
# <mailto:timb@nth-dimension.org.uk>
# <http://www.nth-dimension.org.uk/> / <http://www.machine.org.uk/>

package Fuzzled::Protocol::IMAP;

use 5.008007;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

use Fuzzled::Helper::SharedMemory;
use Fuzzled::Helper;
use Fuzzled::Helper::TCP;
use Fuzzled::Factory::BruteForce;
use Fuzzled::Factory::Repeat;
use Fuzzled::Factory::Replace;
use Fuzzled::Namespace::FormatString;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration use Fuzzled::Protocol::IMAP ':all';
# If you do not need self, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = ('$Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-IMAP/lib/Fuzzled/Protocol/IMAP.pm,v 1.6 2007-09-15 22:14:32 timb Exp $' =~ m/.Header: .* ([\d.]+) .*/)[0];

# Preloaded methods go here.
sub new {
	my $self;
	$self = shift;
	$self = bless({}, $self);
	$self->{'hostname'} = shift;
	$self->{'portnumber'} = shift;
	$self->{'username'} = shift;
	$self->{'password'} = shift;
	$self->{'maximumprocess'} = shift;
	$self->{'debugflag'} = shift;
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::IMAP->new() succeeded\n";
	return $self;
}

sub attack {
	my $self;
	my $tcphandle;
	my $currentprocess;
	my $memoryerror;
	my $consumererror;
	my $processid;
	$self = shift;
	$self->{'helperhandle'} = Fuzzled::Helper->new($self->{'debugflag'});
	$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
	if ($tcphandle == 0) {
		die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
	}
	$tcphandle->close();
	$currentprocess = 0;
	$memoryerror = 0;
	$consumererror = 0;
	while (($currentprocess < $self->{'maximumprocess'}) && ($memoryerror == 0) && ($consumererror == 0)) {
		$self->{'memoryhandle'} = Fuzzled::Helper::SharedMemory->new(4500, "\x03EOD\x04", $self->{'debugflag'});
		if ($self->{'memoryhandle'} == 0) {
			$memoryerror = 1;
		} else {
			$self->{'memoryhandle'}->write("\x03EMPTY\x04");
			$processid = $self->consumer();
			if ($processid < 0) {
				$self->{'memoryhandle'}->dealloc();
				$consumererror = 1;
			} else {
				if ($processid == 0) {
					print "[fuzzled] " . $$ . " finished\n";
					$self->{'memoryhandle'}->dealloc();
					$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::IMAP->attack() succeeded: consumer\n";
					return 2;
				}
				print "[fuzzled] " . $$ . " begat " . $processid . "\n";
				$self->{'memoryhandles'}[$currentprocess] = $self->{'memoryhandle'};
				$currentprocess ++;
			}
		}
	}
	$self->{'maximumprocess'} = $currentprocess;
	if ($self->{'maximumprocess'} > 0) {
		if ($self->producer() > 0) {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::IMAP->attack() succeeded: producer\n";
			return 1;
		}
		$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::IMAP->attack() failed: producer\n";
		return 0;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::IMAP->attack() failed: producer\n";
	return 0;
}

sub producer {
	my $self;
	my @requeststrings;
	my $requesthandle;
	my $requeststring;
	my $overflowhandle;
	my $overflowstring;
	my $formathandle;
	my $formatstring;
	my $fullrequesthandle;
	my $fullrequeststring;
	my $queuedflag;
	my $memoryhandle;
	$self = shift;
	$SIG{'INT'} = sub {
		$queuedflag = 1;
		while ($queuedflag == 1) {
			$queuedflag = 0;
			foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
				if ($memoryhandle->lock() == 1) {
					if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
						$memoryhandle->poke("\x03END\x04");
						$queuedflag = 1;
					} else {
						if ($memoryhandle->peek() ne "\x03END\x04") {
							$queuedflag = 1;
						}
					}
					$memoryhandle->unlock();
				}
				
			}
		}
		die "Fuzzled::Protocol::IMAP->producer() failed: interrupted\n";
	};
	@requeststrings = ["A001 CREATE \x03DATA\x04", "A001 CREATE \"\x03DATA\x04", "FXXZ CHECK \x03DATA\x04", "LIST \x03DATA\x04", "A001 SELECT \x03DATA\x04", "A001 SELECT \"\x03DATA\x04", "A001 EXAMINE \x03DATA\x04", "A001 EXAMINE \"\x03DATA\x04", "A001 DELETE \x03DATA\x04", "A001 DELETE \"\x03DATA\x04", "A001 RENAME \x03DATA\x04", "A001 RENAME \"\x03DATA\x04", "A001 CREATE test\r\nA001 RENAME test \x03DATA\x04", "A001 SUBSCRIBE \x03DATA\x04", "A001 UNSUBSCRIBE \x03DATA\x04", "A001 LIST \x03DATA\x04 aa", "A001 LIST aa \x03DATA\x04", "A001 LIST * \x03DATA\x04", "A001 LSUB aa \x03DATA\x04", "A001 LSUB \x03DATA\x04 aa", "A001 STATUS \x03DATA\x04", "A001 STATUS \"\x03DATA\x04", "A001 STATUS inbox (\x03DATA\x04)", "A001 APPEND \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 SEARCH \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 FETCH \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 FETCH 1:2 \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 STORE \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 STORE 1:2 \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 COPY \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 COPY 1:2 \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 UID \x03DATA\x04", "A001 SELECT \x03DATA\x04\r\nA001 UID FETCH \x03DATA\x04", "A001 UID \x03DATA\x04", "A001 CAPABILITY \x03DATA\x04", "A001 DELETEACL \x03DATA\x04", "A001 DELETEACL \"\x03DATA\x04", "A001 GETACL \x03DATA\x04", "A001 GETACL \"\x03DATA\x04", "A001 LISTRIGHTS \x03DATA\x04", "A001 LISTRIGHTS \"\x03DATA\x04", "A001 MYRIGHTS \x03DATA\x04", "A001 MYRIGHTS \"\x03DATA\x04", "A001 \x03DATA\x04", "A001 \"\x03DATA\x04", "\x03DATA\x04"];
	$requesthandle = Fuzzled::Factory::BruteForce->new(@requeststrings);
	while ($requeststring = $requesthandle->generate()) {
		print "[fuzzled] checking overflows (" . $requeststring . ")\n";
		$overflowhandle = Fuzzled::Factory::Repeat->new("A", 0, 2048);
		while ($overflowstring = $overflowhandle->generate()) {
			$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["\x03DATA\x04", ""]], $overflowstring);
			while ($fullrequeststring = $fullrequesthandle->generate()) {
				$queuedflag = 0;
				while ($queuedflag != 1) {
					foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
						if ($memoryhandle->lock() == 1) {
							if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
								$memoryhandle->poke($fullrequeststring);
								$memoryhandle->unlock();
								$queuedflag = 1;
								last;
							}
							$memoryhandle->unlock();
						}
					}
					if ($queuedflag == 0) {
						$self->{'helperhandle'}->sleep(0.1);
					}
				}
			}
		}
	}
	$requesthandle = Fuzzled::Factory::BruteForce->new(@requeststrings);
	while ($requeststring = $requesthandle->generate()) {
		print "[fuzzled] checking formats (" . $requeststring . ")\n";
		$formathandle = Fuzzled::Factory::BruteForce->new((["1"], Fuzzled::Namespace::FormatString->generate()));
		while ($formatstring = $formathandle->generate()) {
			$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["\x03DATA\x04", ""]], $formatstring);
			while ($fullrequeststring = $fullrequesthandle->generate()) {
				$queuedflag = 0;
				while ($queuedflag != 1) {
					foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
						if ($memoryhandle->lock() == 1) {
							if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
								$memoryhandle->poke($fullrequeststring);
								$memoryhandle->unlock();
								$queuedflag = 1;
								last;
							}
							$memoryhandle->unlock();
						}
					}
					if ($queuedflag == 0) {
						$self->{'helperhandle'}->sleep(0.1);
					}
				}
			}
		}
	}
	$queuedflag = 1;
	while ($queuedflag == 1) {
		$queuedflag = 0;
		foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
			if ($memoryhandle->lock() == 1) {
				if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
					$memoryhandle->poke("\x03END\x04");
					$queuedflag = 1;
				} else {
					if ($memoryhandle->peek() ne "\x03END\x04") {
						$queuedflag = 1;
					}
				}
				$memoryhandle->unlock();
			}
			
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::IMAP->producer() succeeded\n";
	return 1;
}

sub consumer {
	my $self;
	my $processid;
	my $requeststring;
	my $fullrequeststring;
	my $tcphandle;
	$self = shift;
	$processid = fork();
	if ($processid == 0) {
		$SIG{'INT'} = sub {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::IMAP->consumer() interrupted\n";
		};
		while ($self->{'memoryhandle'}->read() ne "\x03END\x04") {
			if ($self->{'memoryhandle'}->lock() == 1) {
				$requeststring = $self->{'memoryhandle'}->peek();
				if ($requeststring eq "\x03END\x04") {
					$self->{'memoryhandle'}->unlock();
				} else {
					if ($requeststring eq "\x03EMPTY\x04") {
						$self->{'memoryhandle'}->unlock();
						$self->{'helperhandle'}->sleep(1);
					} else {
						$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
						if ($tcphandle == 0) {
							die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
						}
						$self->{'memoryhandle'}->poke("\x03EMPTY\x04");
						$self->{'memoryhandle'}->unlock();
						$fullrequeststring = "A001 LOGIN " . $self->{'username'} . " " . $self->{'password'} . "\r\n";
						$fullrequeststring = $fullrequeststring . $requeststring . "\r\n";
						$tcphandle->write($fullrequeststring);
						print "[fuzzled] " . $requeststring . "\n";
						$self->{'helperhandle'}->log("IMAP." . $$, "wrote " . $fullrequeststring . "\n");
					}
				}		
			} else {
				$self->{'helperhandle'}->sleep(1);
			}
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::IMAP->consumer() succeeded\n";
	return $processid;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

Perl extension for Fuzzled to fuzz IMAP

=head1 SYNOPSIS

  use Fuzzled::Protocol::IMAP;

  my $imaptesthandle;

  $imaptesthandle = Fuzzled::Protocol::IMAP->new("service.example.org", 143, "username", "password", 1, 1);
  $imaptesthandle->attack();

=head1 DESCRIPTION

C<Fuzzled::Protocol::IMAP> is a Perl extension for Fuzzled to fuzz IMAP.  The following methods are available:

=over 4

=item Fuzzled::Protocol::IMAP->new(<hostname>, <portnumber>, <username>, <password>, <maximumprocess>, <debugflag>)

Constructs a new C<Fuzzled::Protocol::IMAP> object describing the connection to be fuzzed.

=item Fuzzled::Protocol::IMAP->attack()

Carries out the fuzzing.

Returns 1 

=head1 SEE ALSO

=head1 AUTHOR

(c) Tim Brown, 2007
<mailto:timb@nth-dimension.org.uk>

=head1 COPYRIGHT AND LICENSE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with self program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

=cut
