# $Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-MySQLAuthenticate/lib/Fuzzled/Protocol/MySQLAuthenticate.pm,v 1.1 2007-10-02 23:53:57 timb Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with self program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# (c) Tim Brown, 2006
# <mailto:timb@nth-dimension.org.uk>
# <http://www.nth-dimension.org.uk/> / <http://www.machine.org.uk/>

package Fuzzled::Protocol::MySQLAuthenticate;

use 5.008007;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

use Fuzzled::Helper::SharedMemory;
use Fuzzled::Helper;
use Fuzzled::Helper::TCP;
use Fuzzled::PacketParse::Separate;
use Fuzzled::Factory::Repeat;
use Fuzzled::Factory::Increment;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration use Fuzzled::Protocol::MySQLAuthenticate ':all';
# If you do not need self, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = ('$Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-MySQLAuthenticate/lib/Fuzzled/Protocol/MySQLAuthenticate.pm,v 1.1 2007-10-02 23:53:57 timb Exp $' =~ m/.Header: .* ([\d.]+) .*/)[0];

# Preloaded methods go here.
sub new {
	my $self;
	$self = shift;
	$self = bless({}, $self);
	$self->{'hostname'} = shift;
	$self->{'portnumber'} = shift;
	$self->{'maximumprocess'} = shift;
	$self->{'debugflag'} = shift;
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::MySQLAuthenticate->new() succeeded\n";
	return $self;
}

sub attack {
	my $self;
	my $tcphandle;
	my $currentprocess;
	my $memoryerror;
	my $consumererror;
	my $processid;
	$self = shift;
	$self->{'helperhandle'} = Fuzzled::Helper->new($self->{'debugflag'});
	$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
	if ($tcphandle == 0) {
		die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
	}
	$tcphandle->close();
	$currentprocess = 0;
	$memoryerror = 0;
	$consumererror = 0;
	while (($currentprocess < $self->{'maximumprocess'}) && ($memoryerror == 0) && ($consumererror == 0)) {
		$self->{'memoryhandle'} = Fuzzled::Helper::SharedMemory->new(6500, "\x03EOD\x04", $self->{'debugflag'});
		if ($self->{'memoryhandle'} == 0) {
			$memoryerror = 1;
		} else {
			$self->{'memoryhandle'}->write("\x03EMPTY\x04");
			$processid = $self->consumer();
			if ($processid < 0) {
				$self->{'memoryhandle'}->dealloc();
				$consumererror = 1;
			} else {
				if ($processid == 0) {
					print "[fuzzled] " . $$ . " finished\n";
					$self->{'memoryhandle'}->dealloc();
					$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::MySQLAuthenticate->attack() succeeded: consumer\n";
					return 2;
				}
				print "[fuzzled] " . $$ . " begat " . $processid . "\n";
				$self->{'memoryhandles'}[$currentprocess] = $self->{'memoryhandle'};
				$currentprocess ++;
			}
		}
	}
	$self->{'maximumprocess'} = $currentprocess;
	if ($self->{'maximumprocess'} > 0) {
		if ($self->producer() > 0) {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::MySQLAuthenticate->attack() succeeded: producer\n";
			return 1;
		}
		$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::MySQLAuthenticate->attack() failed: producer\n";
		return 0;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::MySQLAuthenticate->attack() failed: producer\n";
	return 0;
}

sub producer {
	my $self;
	my $queuedflag;
	my $memoryhandle;
	my $commandcodehandle;
	my $commandcodestring;
	my $overflow1handle;
	my $overflow1string;
	my $overflow2handle;
	my $overflow2string;
	my $usernamehandle;
	my $usernamestring;
	my $overflow3handle;
	my $overflow3string;
	my $databasenamehandle;
	my $databasenamestring;
	my $commandcode1handle;
	my $commandcode1character;
	my $commandcode2handle;
	my $commandcode2character;
	my $parsinghandle;
	my $parsingstring;
	$self = shift;
	$SIG{'INT'} = sub {
		$queuedflag = 1;
		while ($queuedflag == 1) {
			$queuedflag = 0;
			foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
				if ($memoryhandle->lock() == 1) {
					if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
						$memoryhandle->poke("\x03END\x04");
						$queuedflag = 1;
					} else {
						if ($memoryhandle->peek() ne "\x03END\x04") {
							$queuedflag = 1;
						}
					}
					$memoryhandle->unlock();
				}
				
			}
		}
		die "Fuzzled::Protocol::MySQLAuthenticate->producer() failed: interrupted\n";
	};
	# based on Fuzzled::PacketParse::Separate output
	# <2 bytes><null>
	# <string><null>
	# <string><null>
	# <username><null>
	# <string>
	# <databasename><null>
	print "[fuzzled] checking overflows\n";
	$commandcodehandle = Fuzzled::Factory::Repeat->new("A", 0, 1024);
	while ($commandcodestring = $commandcodehandle->generate()) {
		$overflow1handle = Fuzzled::Factory::Repeat->new("A", 0, 1024);
		while ($overflow1string = $overflow1handle->generate()) {
			$overflow2handle = Fuzzled::Factory::Repeat->new("A", 0, 1024);
			while ($overflow2string = $overflow2handle->generate()) {
				$usernamehandle = Fuzzled::Factory::Repeat->new("A", 0, 1024);
				while ($usernamestring = $usernamehandle->generate()) {
					$overflow3handle = Fuzzled::Factory::Repeat->new("A", 0, 1024);
					while ($overflow3string = $overflow3handle->generate()) {
						$databasenamehandle = Fuzzled::Factory::Repeat->new("A", 0, 1024);
						while ($databasenamestring = $databasenamehandle->generate()) {
							$queuedflag = 0;
							while ($queuedflag != 1) {
								foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
									if ($memoryhandle->lock() == 1) {
										if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
											$memoryhandle->poke($commandcodestring . "\x00" . $overflow1string . "\x00", $overflow2string . "\x00" . $usernamestring . "\x00" . $overflow3string . $databasenamestring . "\x00");
											$memoryhandle->unlock();
											$queuedflag = 1;
											last;
										}
										$memoryhandle->unlock();
									}
								}
								if ($queuedflag == 0) {
									$self->{'helperhandle'}->sleep(0.1);
								}
							}
						}
					}
				}
			}
		}
	}
	print "[fuzzled] checking parsing\n";
	$commandcode1handle = Fuzzled::Factory::Increment->new(0, 255, 1);
	while ($commandcode1character = $commandcode1handle->generate()) {
		$commandcode2handle = Fuzzled::Factory::Increment->new(0, 255, 1);
		while ($commandcode2character = $commandcode2handle->generate()) {
			$parsinghandle = Fuzzled::Factory::Repeat->new("A\x00", 0, 1024);
			while ($parsingstring = $parsinghandle->generate()) {
				$queuedflag = 0;
				while ($queuedflag != 1) {
					foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
						if ($memoryhandle->lock() == 1) {
							if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
								$memoryhandle->poke(pack("CC", $commandcode1character, $commandcode2character) . $parsingstring);
								$memoryhandle->unlock();
								$queuedflag = 1;
								last;
							}
							$memoryhandle->unlock();
						}
					}
					if ($queuedflag == 0) {
						$self->{'helperhandle'}->sleep(0.1);
					}
				}
			}
		}
	}
	$queuedflag = 1;
	while ($queuedflag == 1) {
		$queuedflag = 0;
		foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
			if ($memoryhandle->lock() == 1) {
				if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
					$memoryhandle->poke("\x03END\x04");
					$queuedflag = 1;
				} else {
					if ($memoryhandle->peek() ne "\x03END\x04") {
						$queuedflag = 1;
					}
				}
				$memoryhandle->unlock();
			}
			
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::MySQLAuthenticate->producer() succeeded\n";
	return 1;
}

sub consumer {
	my $self;
	my $processid;
	my $requeststring;
	my $fullrequeststring;
	my $tcphandle;
	my $responsestring;
	$self = shift;
	$processid = fork();
	if ($processid == 0) {
		$SIG{'INT'} = sub {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::MySQLAuthenticate->consumer() interrupted\n";
		};
		while ($self->{'memoryhandle'}->read() ne "\x03END\x04") {
			if ($self->{'memoryhandle'}->lock() == 1) {
				$requeststring = $self->{'memoryhandle'}->peek();
				if ($requeststring eq "\x03END\x04") {
					$self->{'memoryhandle'}->unlock();
				} else {
					if ($requeststring eq "\x03EMPTY\x04") {
						$self->{'memoryhandle'}->unlock();
						$self->{'helperhandle'}->sleep(1);
					} else {
						$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
						if ($tcphandle == 0) {
							die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
						}
						$self->{'memoryhandle'}->poke("\x03EMPTY\x04");
						$self->{'memoryhandle'}->unlock();
						$fullrequeststring = $requeststring;
						$tcphandle->write($fullrequeststring);
						print "[fuzzled] wrote " . Fuzzled::PacketParse::Separate->parse($fullrequeststring, 0, "\x00") . "\n";
						$self->{'helperhandle'}->log("MySQLAuthenticate." . $$, "wrote " . $fullrequeststring . "\n");
						do {
							$responsestring = $tcphandle->read();
							print "[fuzzled] read " . Fuzzled::PacketParse::Separate->parse($responsestring, 0, "\x00") . "\n";
							$self->{'helperhandle'}->log("MySQLAuthenticate." . $$, "read " . $responsestring . "\n");
						} while ($responsestring ne "");
						$tcphandle->close();
					}
				}
			} else {
				$self->{'helperhandle'}->sleep(1);
			}
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::MySQLAuthenticate->consumer() succeeded\n";
	return $processid;
	return $processid;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

Perl extension for Fuzzled to fuzz MySQL authentication

=head1 SYNOPSIS

  use Fuzzled::Protocol::MySQLAuthenticate;

  my $mysqlauthenticatetesthandle;

  $mysqlauthenticatetesthandle = Fuzzled::Protocol::MySQLAuthenticate->new("service.example.org", 3306, 1, 1);
  $mysqlauthenticatetesthandle->attack();

=head1 DESCRIPTION

C<Fuzzled::Protocol::MySQLAuthenticate> is a Perl extension for Fuzzled to fuzz MySQL authentication.  The following methods are available:

=over 4

=item Fuzzled::Protocol::MySQLAuthenticate->new(<hostname>, <portnumber>, <maximumprocess>, <debugflag>)

Constructs a new C<Fuzzled::Protocol::MySQLAuthenticate> object describing the connection to be fuzzed.

=item Fuzzled::Protocol::MySQLAuthenticate->attack()

Carries out the fuzzing.

Returns 1 

=head1 SEE ALSO

=head1 AUTHOR

(c) Tim Brown, 2006
<mailto:timb@nth-dimension.org.uk>

=head1 COPYRIGHT AND LICENSE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with self program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

=cut
