#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with self program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# (c) Nico Leidecker, 2007
# <mailto:nico@nth-dimension.org.uk>
# <http://www.nth-dimension.org.uk/> / <http://www.leidecker.info/>

package Fuzzled::Protocol::NNTP;

use 5.008007;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

use Fuzzled::Helper::SharedMemory;
use Fuzzled::Helper;
use Fuzzled::Helper::TCP;
use Fuzzled::Factory::BruteForce;
use Fuzzled::Factory::Repeat;
use Fuzzled::Factory::Replace;
use Fuzzled::Namespace::FormatString;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration use Fuzzled::Protocol::NNTP ':all';
# If you do not need self, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = ('$Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-NNTP/lib/Fuzzled/Protocol/NNTP.pm,v 1.4 2007-09-15 22:14:32 timb Exp $' =~ m/.Header: .* ([\d.]+) .*/)[0];

# Preloaded methods go here.
sub new {
	my $self;
	$self = shift;
	$self = bless({}, $self);
	$self->{'hostname'} = shift;
	$self->{'portnumber'} = shift;
	$self->{'maximumprocess'} = shift;
	$self->{'debugflag'} = shift;
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::NNTP->new() succeeded\n";
	return $self;
}

sub attack {
	my $self;
	my $tcphandle;
	my $currentprocess;
	my $memoryerror;
	my $consumererror;
	my $processid;
	$self = shift;
	$self->{'helperhandle'} = Fuzzled::Helper->new($self->{'debugflag'});
	$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
	if ($tcphandle == 0) {
		die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
	}
	$tcphandle->close();
	$currentprocess = 0;
	$memoryerror = 0;
	$consumererror = 0;
	while (($currentprocess < $self->{'maximumprocess'}) && ($memoryerror == 0) && ($consumererror == 0)) {
		$self->{'memoryhandle'} = Fuzzled::Helper::SharedMemory->new(4500, "\x03EOD\x04", $self->{'debugflag'});
		if ($self->{'memoryhandle'} == 0) {
			$memoryerror = 1;
		} else {
			$self->{'memoryhandle'}->write("\x03EMPTY\x04");
			$processid = $self->consumer();
			if ($processid < 0) {
				$self->{'memoryhandle'}->dealloc();
				$consumererror = 1;
			} else {
				if ($processid == 0) {
					print "[fuzzled] " . $$ . " finished\n";
					$self->{'memoryhandle'}->dealloc();
					$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::NNTP->attack() succeeded: consumer\n";
					return 2;
				}
				print "[fuzzled] " . $$ . " begat " . $processid . "\n";
				$self->{'memoryhandles'}[$currentprocess] = $self->{'memoryhandle'};
				$currentprocess ++;
			}
		}
	}
	$self->{'maximumprocess'} = $currentprocess;
	if ($self->{'maximumprocess'} > 0) {
		if ($self->producer() > 0) {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::NNTP->attack() succeeded: producer\n";
			return 1;
		}
		$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::NNTP->attack() failed: producer\n";
		return 0;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::NNTP->attack() failed: producer\n";
	return 0;
}

sub producer {
	my $self;
	my @requeststrings;
	my $requesthandle;
	my $requeststring;
	my $overflowhandle;
	my $overflowstring;
	my $formathandle;
	my $formatstring;
	my $fullrequesthandle;
	my $fullrequeststring;
	my $queuedflag;
	my $memoryhandle;
	$self = shift;
	$SIG{'INT'} = sub {
		$queuedflag = 1;
		while ($queuedflag == 1) {
			$queuedflag = 0;
			foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
				if ($memoryhandle->lock() == 1) {
					if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
						$memoryhandle->poke("\x03END\x04");
						$queuedflag = 1;
					} else {
						if ($memoryhandle->peek() ne "\x03END\x04") {
							$queuedflag = 1;
						}
					}
					$memoryhandle->unlock();
				}
				
			}
		}
		die "Fuzzled::Protocol::NNTP->producer() failed: interrupted\n";
	};
	@requeststrings = ["LIST \x03DATA\x04", "ARTICLE <\x03DATA\x04>", "ARTICLE <\x03DATA\x04\@aa>", "ARTICLE <aa\@\x03DATA\x04>", "IHAVE <\x03DATA\x04>", "IHAVE <\x03DATA\x04\@aa>", "IHAVE <aa\@\x03DATA\x04>", "BODY \x03DATA\x04", "GROUP \x03DATA\x04", "HEAD \x03DATA\x04", "STAT \x03DATA\x04", "NEWSGROUPS \x03DATA\x04", "NEWSGROUPS \x03DATA\x04 000000", "NEWSGROUPS \x03DATA\x04 \x03DATA\x04", "NEWSGROUPS 000000 \x03DATA\x04", "NEWNEWS * 000000 \x03DATA\x04", "NEWNEWS * \x03DATA\x04 000000", "NEWNEWS * \x03DATA\x04 \x03DATA\x04", "\x03DATA\x04"];
	$requesthandle = Fuzzled::Factory::BruteForce->new(@requeststrings);
	while ($requeststring = $requesthandle->generate()) {
		print "[fuzzled] checking overflows (" . $requeststring . ")\n";
		$overflowhandle = Fuzzled::Factory::Repeat->new("A", 0, 2048);
		while ($overflowstring = $overflowhandle->generate()) {
			$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["\x03DATA\x04", ""]], $overflowstring);
			while ($fullrequeststring = $fullrequesthandle->generate()) {
				$queuedflag = 0;
				while ($queuedflag != 1) {
					foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
						if ($memoryhandle->lock() == 1) {
							if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
								$memoryhandle->poke($fullrequeststring);
								$memoryhandle->unlock();
								$queuedflag = 1;
								last;
							}
							$memoryhandle->unlock();
						}
					}
					if ($queuedflag == 0) {
						$self->{'helperhandle'}->sleep(0.1);
					}
				}
			}
		}
	}
	$requesthandle = Fuzzled::Factory::BruteForce->new(@requeststrings);
	while ($requeststring = $requesthandle->generate()) {
		print "[fuzzled] checking formats (" . $requeststring . ")\n";
		$formathandle = Fuzzled::Factory::BruteForce->new((["1"], Fuzzled::Namespace::FormatString->generate()));
		while ($formatstring = $formathandle->generate()) {
			$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["\x03DATA\x04", ""]], $formatstring);
			while ($fullrequeststring = $fullrequesthandle->generate()) {
				$queuedflag = 0;
				while ($queuedflag != 1) {
					foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
						if ($memoryhandle->lock() == 1) {
							if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
								$memoryhandle->poke($fullrequeststring);
								$memoryhandle->unlock();
								$queuedflag = 1;
								last;
							}
							$memoryhandle->unlock();
						}
					}
					if ($queuedflag == 0) {
						$self->{'helperhandle'}->sleep(0.1);
					}
				}
			}
		}
	}
	$queuedflag = 1;
	while ($queuedflag == 1) {
		$queuedflag = 0;
		foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
			if ($memoryhandle->lock() == 1) {
				if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
					$memoryhandle->poke("\x03END\x04");
					$queuedflag = 1;
				} else {
					if ($memoryhandle->peek() ne "\x03END\x04") {
						$queuedflag = 1;
					}
				}
				$memoryhandle->unlock();
			}
			
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::NNTP->producer() succeeded\n";
	return 1;
}

sub consumer {
	my $self;
	my $processid;
	my $requeststring;
	my $fullrequeststring;
	my $tcphandle;
	$self = shift;
	$processid = fork();
	if ($processid == 0) {
		$SIG{'INT'} = sub {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::NNTP->consumer() interrupted\n";
		};
		while ($self->{'memoryhandle'}->read() ne "\x03END\x04") {
			if ($self->{'memoryhandle'}->lock() == 1) {
				$requeststring = $self->{'memoryhandle'}->peek();
				if ($requeststring eq "\x03END\x04") {
					$self->{'memoryhandle'}->unlock();
				} else {
					if ($requeststring eq "\x03EMPTY\x04") {
						$self->{'memoryhandle'}->unlock();
						$self->{'helperhandle'}->sleep(1);
					} else {
						$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
						if ($tcphandle == 0) {
							die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
						}
						$self->{'memoryhandle'}->poke("\x03EMPTY\x04");
						$self->{'memoryhandle'}->unlock();
						$fullrequeststring = $requeststring . "\r\n";
						
						$tcphandle->write($fullrequeststring);
						print "[fuzzled] " . $requeststring . "\n";
						$self->{'helperhandle'}->log("NNTP." . $$, "wrote " . $fullrequeststring . "\n");
					}
				}		
			} else {
				$self->{'helperhandle'}->sleep(1);
			}
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::NNTP->consumer() succeeded\n";
	return $processid;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

Perl extension for Fuzzled to fuzz NNTP

=head1 SYNOPSIS

  use Fuzzled::Protocol::NNTP;

  my $nntphandle;

  my $process_num = 1;
  my $debug_flag = 1;

  $nntphandle = Fuzzled::Protocol::NNTP->new("service.example.org", 119, 1, 1);
  $nntphandle->attack();

=head1 DESCRIPTION

C<Fuzzled::Protocol::NNTP> is a Perl extension for Fuzzled to fuzz NNTP.  The following methods are available:

=over 4

=item Fuzzled::Protocol::NNTP->new(<hostname>, <portnumber>, <maximumprocess>, <debugflag>)

Constructs a new C<Fuzzled::Protocol::NNTP> object describing the connection to be fuzzed.

=item Fuzzled::Protocol::NNTP->attack()

Carries out the fuzzing.

Returns 1 

=head1 SEE ALSO

=head1 AUTHOR

(c) Nico Leidecker, 2007
<mailto:nico@nth-dimension.org.uk>

=head1 COPYRIGHT AND LICENSE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with self program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

=cut
